//****************************************************
//***** m6955/ENCODER/LCD APP    *********************
//*************************** By Minarai Shokunin ****
//*************************** 2014/02/16          ****
//****************************************************

#include "p24FJ64GA002.h"
#include "spilcd.h"
#include "m6955_2.h"

// ************ CONFIGRATION BITS ********************
_CONFIG1(JTAGEN_OFF & GCP_OFF & GWRP_OFF & BKBUG_OFF & COE_OFF & ICS_PGx1 & FWDTEN_OFF )
_CONFIG2(IESO_OFF & FNOSC_FRCPLL & FCKSM_CSDCMD & OSCIOFNC_ON & IOL1WAY_OFF & I2C1SEL_PRI & POSCMOD_NONE)


extern unsigned char VirtualScreen[6][128];

// ************ prototypes ***************************
extern void SPILCD_init();
extern void WriteData(int,int);
extern void Copy2SPILCD(void);
extern void GR_SCALE(int);
extern void GR_FILL(unsigned char);
extern void GR_PSET(int,int,unsigned char);
extern void GR_LINE(int,int,int,int,unsigned char);
extern void GR_PUTCH(int,int,unsigned char, unsigned char);
extern void GR_BOXFILL(int,int,int,int,unsigned char);
extern void GR_BOX(int,int,int,int,unsigned char);
extern void GR_PUTNUM(int,int,int, unsigned char);
extern void GR_PUTSTR(int,int,unsigned char *, unsigned char);
extern void I2c_Reg_Write(unsigned int,unsigned char *,unsigned int );
extern void I2c_Reg_Read(unsigned int,unsigned char *,unsigned int);

extern void m6955_init(void);
extern void m6955_setband(int);
extern void m6955_setfreq(long);
extern char m6955_getreg(char);

void wait_some(int);
void Put_Freq(int,int,long, unsigned char);
void Disp_RSSI(void);
void scan_band(void);
void disp_main(void);

void __attribute__((interrupt, auto_psv)) _T1Interrupt(void);

#define LED_R 	LATBbits.LATB6
#define LED_G 	LATBbits.LATB7
#define LED_B 	LATBbits.LATB8
#define SW 		PORTBbits.RB9
#define ENC_A 	PORTBbits.RB10
#define ENC_B 	PORTBbits.RB5

#define CW		1
#define	CCW		-1
#define HOLD	0
#define ON		1
#define OFF		0
#define TH_SLOW	2500


//************************ LW     MW    SW1    SW2     SW3     SW4     SW5     SW6    SW7      SW8     SW9      SW10     SW12     SW13       FM2
const long BAND_FREQL[] = {150 ,  522, 4700,   3200,   4700,   5700,   6800,   9200,  11400,   13500,  15000,   17400,   21400,   11400,   76000,0,0};
long BAND_FREQM[]       = {150 ,  810, 4700,   3200,   4700,   5700,   6800,   9200,  11400,   13500,  15000,   17400,   21400,   11400,   81300,0,0};
const long BAND_FREQH[] = {285 , 1620, 5600,   4100,   5600,   6400,   7600,  10000,  12200,   14300,  15900,   17900,   21900,   17900,   108000,0,0};
const long BAND_STEP[] =  {  5,    9,    5,    5,      5,      5,      5,      5,      5,       5,      5,       5,       5,       5,      100,0,0};
const int BAND_SCALE[] =  {  3,    3,    6,    6,      6,      6,      6,      6,      6,       6,      6,       6,       6,       6,      3,0,0};
const char BAND_NAME[][7]= {"LW","MW","SW4.7","SW3.2","SW4.7","SW5.7","SW6.8","SW9.2","SW11.4","SW13.5","SW15.0","SW17.4","SW21.4","SW11.4","FM","SCAN"};



int nEncoder_state;
int nEncoder_dir;
int nEncoder_count;
int nEncoder_sw;
int nEncoder_data_valid;
int nEncoder_pushed = 0;

int nGlBand = AMMODE;		// FM mode
long lGlFreq	= 810;	// 76MHz
int nGlRSSI = 0;
int nGlCNR = 0;
int nGlBand_bkup;

// *********************************************
// ************ MAIN ***************************
// *********************************************

int main(void)
{
	int i;
	int freq_scale = 1;
	// ************ CPU Initialize **********
	
	_RCDIV = 0;						// PostScaler 32MHz
	AD1PCFG = 0xffff;				// All digital
	TRISB = 0b0000011000110000;			// PORTB 4,5,9 are input / others outout
	PORTB = 0xffff;
	
	nEncoder_state = 0;
	nEncoder_count;
	nEncoder_dir = HOLD;
	nEncoder_data_valid = 0;

	SPILCD_init();

	disp_main();


	// *********** M6955 initialize *******************
	m6955_init();

	// *********** Interval timer Itinialize ***************
    T1CONbits.TCKPS 	= 1;   // Timer1 Pre scaler 1:8
	PR1 				= 200; // 100us interval timer
    T1CONbits.TON   	= 1;    // Timer1 ON
 	IEC0bits.T1IE = 1;  // Timer1 interrupt enable

	// *********** Main loop start here **************
	m6955_setband(nGlBand);
	m6955_setfreq(lGlFreq);	
	Put_Freq(10,32,lGlFreq,1);
	Copy2SPILCD();
	wait_some(1000);

	//*************************************************
	//***** MAIN LOOP *********************************
	//*************************************************

	while (1) {

		// ********** GR[_] *************

		if (nEncoder_data_valid == 1) {			
			// ********** ܂܉񂵂ꍇ̓oh؂ւ *********
			if (SW == ON) {
				GR_FILL(0x00);
				GR_SCALE(1);
				GR_PUTSTR(10,0,"AITENDO M6955",1);
				GR_SCALE(2);
				BAND_FREQM[nGlBand] = lGlFreq;		// ******* ̃oh̎gobNAbv **********
				if (nEncoder_dir == CW) {			// ***E񂵑Ή *******
					nGlBand ++;
					if (nGlBand > SCANMODE) {
						nGlBand = 0;
					}
				}else{								// ***񂵑Ή ******
					nGlBand --;
					if (nGlBand < 0 ) {
						nGlBand = SCANMODE;
					}
				}
				
				GR_PUTSTR(10,10,BAND_NAME[nGlBand],1);
				GR_BOXFILL(10,30,110,20,0);
				if (nGlBand <=FMMODE) {
					lGlFreq = BAND_FREQM[nGlBand];
					m6955_setband(nGlBand);
					m6955_setfreq(lGlFreq);
					Put_Freq(10,30,lGlFreq,1);
				}	
				Copy2SPILCD();
			}else{
			// ********** 񂵂ꍇ̓`[jO *********
				freq_scale = 1;
				if (nEncoder_count < TH_SLOW) {
					freq_scale = BAND_SCALE[nGlBand];;
				}
				if (nEncoder_dir == CW) {
					lGlFreq += BAND_STEP[nGlBand]*freq_scale;
					if (lGlFreq > BAND_FREQH[nGlBand]) {
						lGlFreq -= BAND_STEP[nGlBand]*freq_scale;
					}
				}else{
					lGlFreq -= BAND_STEP[nGlBand]*freq_scale;
					if (lGlFreq < BAND_FREQL[nGlBand]) {		
						lGlFreq += BAND_STEP[nGlBand]*freq_scale;
					}
				}
				m6955_setband(nGlBand);
				m6955_setfreq(lGlFreq);
				Put_Freq(10,30,lGlFreq,1);
				Copy2SPILCD();
			}
			nEncoder_data_valid = 0;
		}
		// ********** GR[_ *************
		if (nEncoder_pushed == 1) {
			if (nGlBand == SCANMODE) {
				nGlBand = nGlBand_bkup;
				scan_band();							//******** 	ohXLs *******
				nEncoder_pushed = 0;
				GR_BOXFILL(0,0,128,16,1);
				GR_PUTSTR(0,0,"PUSH SW",0);				
				Copy2SPILCD();				
				while (nEncoder_pushed == 0){};			//********* SW҂Ďgɖ߂
				m6955_setband(nGlBand);
				m6955_setfreq(lGlFreq);
				disp_main();
			}
			nEncoder_pushed = 0;
		}
		// ********** RSSI\ *************
		if ((nEncoder_data_valid != 1)&&(nEncoder_pushed != 1)) {
			Disp_RSSI();
		} 
	}	
}

//********** Simple wait loop *************
void wait_some(int t)
{
	int i,j;
	for (i=0;i<t;i++) { 		for (j=0;j<1000;j++) { 		} 	} } //********* main display *********** void disp_main(void) { 	GR_FILL(0x00); 	GR_SCALE(1); 	GR_PUTSTR(10,0,"AITENDO M6955",1); 	GR_SCALE(2); 	GR_PUTSTR(10,10,BAND_NAME[nGlBand],1); 	GR_BOXFILL(10,30,110,20,0); 	Put_Freq(10,30,lGlFreq,1); } //********* scan routine *********** void scan_band(void) { 	int level,wk,i,j; 	int pgalevel_rf; 	int pgalevel_if;  	GR_FILL(0x00); 	GR_SCALE(2);  	for (i=0;i<128;i++) { 			lGlFreq = BAND_FREQL[nGlBand]+i*BAND_STEP[nGlBand];  			m6955_setband(nGlBand); 			m6955_setfreq(lGlFreq); 			GR_BOXFILL(0,0,120,20,0); 			Put_Freq(0,0,lGlFreq,1); 			wait_some(700); 			if (nGlBand == FMMODE) { 				wk = m6955_getreg(0x18); 				pgalevel_rf = (wk >> 5)& 0x07;
				pgalevel_if = (wk >> 2)& 0x07;
				nGlRSSI = m6955_getreg(0x1b) & 0x7f;
				nGlRSSI = 123 - nGlRSSI - 6*pgalevel_rf - 6*pgalevel_if;
				nGlCNR = m6955_getreg(0x17) & 0x3f;	//**** FM CNR ********
				level = nGlRSSI/4;
			}else{
				nGlCNR = (m6955_getreg(0x16) & 0x3f)*2;	//**** AM CNR ********
				level = nGlCNR;
			}
			m6955_getreg(0x00);
			m6955_setband(nGlBand);
			m6955_setfreq(lGlFreq);
		if (level >46) {level = 46;}
		GR_LINE(i,47-level,i,47,1);	
		Copy2SPILCD();
		if (SW==1) {			//*** XCb`ŃLZ *****
			GR_BOXFILL(0,0,128,16,1);
			GR_PUTSTR(0,0,"CANCEL",0);	
			Copy2SPILCD();
			while (SW==1) {};
			break;
		}
	} 
	lGlFreq = BAND_FREQM[nGlBand];
}
//********** RSSI\Tu *************
void Disp_RSSI(void) {
	static long cnt =0;
	int pgalevel_rf;
	int pgalevel_if;
	unsigned char wk;

	cnt++;
	if (cnt>500000) {
		cnt =0;
		wk = m6955_getreg(0x18);
		pgalevel_rf = (wk >> 5)& 0x07;
		pgalevel_if = (wk >> 2)& 0x07;
		nGlRSSI = m6955_getreg(0x1b) & 0x7f;
		nGlRSSI = 123 - nGlRSSI - 6*pgalevel_rf - 6*pgalevel_if;
		if (nGlBand == FMMODE) {
			nGlCNR = m6955_getreg(0x17) & 0x3f;	//**** FM CNR ********
		}else{
			nGlCNR = (m6955_getreg(0x16) & 0x3f)*2;	//**** AM CNR ********
		}
		m6955_getreg(0x00);
		GR_SCALE(1);
		GR_BOXFILL(0,0,128,10,0);
		GR_PUTSTR(0,0,"RSSI",1);
		GR_PUTSTR(64,0,"CNR",1);
		GR_BOXFILL(8*4+2,0,nGlRSSI/4+1,10,1);
		GR_BOXFILL(64+8*3+2,0,nGlCNR/2+1,10,1);
		Copy2SPILCD();
		GR_SCALE(2);
	}
}

//********** g\Tu *************
void Put_Freq(int x,int y,long num, unsigned char cl)
{
	GR_BOXFILL(x,y,128,20,0);
	switch (nGlBand) {
		case LWMODE :GR_PUTNUM(x,y,num,cl);
					GR_PUTSTR(x+16*4,y,"KHZ",cl);
					break;
		case AMMODE :GR_PUTNUM(x,y,num,cl);
					GR_PUTSTR(x+16*4,y,"KHZ",cl);
					break;
		case FMMODE :GR_PUTNUM(x-16*2,y,num/1000,cl);
					 GR_PUTCH(x+16*2,y,'.',cl);
					 GR_PUTCH(x+16*3,y,(num % 1000)/100 + 0x30,cl);
					 GR_PUTSTR(x+16*4,y,"MHZ",cl);
					break;
		default:	
					 GR_PUTNUM(x-16*3+4,y,num/1000,cl);
					 GR_PUTCH(x+16*1+4,y,'.',cl);
					 GR_PUTCH(x+16*2+4,y,(num % 1000)/100 + 0x30,cl);
					 GR_PUTCH(x+16*3+4,y,(num % 100)/10 + 0x30,cl);
					 GR_PUTCH(x+16*4+4,y,(num % 10) + 0x30,cl);
					 GR_PUTSTR(x+16*5+4,y,"MHZ",cl);
					break;
	}
}

// *********************************************
// ************ Timer Interrupt ****************
// *********************************************

void __attribute__((interrupt, auto_psv)) _T1Interrupt(void) {
	static int status_a=0;
	static int status_sw=0;
	static int count = 0;
	int rising;

	if (count < 20000) {count++;}	// **** 	]xopJE^ ****

	status_a = ((status_a <<1) | ENC_A) & 0x0007; // ***@`^Oh~p *******
	status_sw = ((status_sw <<1) | SW) & 0x0007; // ***@`^Oh~p *******

	// **** RO܂͂PωƂ݂ȂB *****
	rising = 0;

	if (status_a == 0x00) {
		nEncoder_state = 0;
	}
	if (status_a ==0x07) {
		if (nEncoder_state == 0) {
			nEncoder_state = 1;
			rising = 1;				// *** 	0PωŁ@オtO@Ă ******
		}
	}
	
	if (status_sw == 0x07) {		// *** XCb`Ă nEncoder_sw tOĂ ***
		LED_R=0;LED_G=1;LED_B=1;	//  *** LEDԂɂ ***
		if (nEncoder_sw == OFF) {
			nGlBand_bkup = nGlBand;				// ****SCAN[hpɃobNAbv ****
		}
		nEncoder_sw = ON;
	}

	if (status_sw == 0x00) {		// *** ꂽ ***
		if (nEncoder_sw == ON) {
			nEncoder_sw = OFF;
			nEncoder_pushed = 1;
			LED_R=1;LED_G=1;LED_B=1;	//  *** SWꂽLED ***
		}
	}

	// ****** A オ̂Ƃ B̏ԂƑx nEncoder_xx ɔf	@*****

	if (rising == 1) {
		if (nEncoder_data_valid == 0) {	   // *** ʂǂݎ܂ŕێ *****
			nEncoder_data_valid = 1;
			nEncoder_count = count;
			if (ENC_B == 0) {				//*** v͐ ****
				nEncoder_dir = CW;
				if (SW == 0) {LED_B = 0;LED_G=1;LED_R=1;}
			}else{
				nEncoder_dir = CCW;			//*** v͗Γ_****
				if (SW == 0) {LED_G = 0;LED_B=1;LED_R=1;}
			}
		}
		count = 0;
	}	
	// *** 荞ݏI@*********
	
    IFS0bits.T1IF = 0; 		// Timer 1 Interrupt flag clear
}
